#include "catch.h"

#include "EdgeT.h"
#include "NodeT.h"

#include <cmath>

#define ADMISS_ERR 1e-20

bool approx(double a, double b) {
  return std::abs(a - b) < ADMISS_ERR;
}

TEST_CASE( "tests for PointT" , "[PointT]" ) {
  NodeT n1(0.0, 0.0, JUNCTION);
  NodeT n2(1.0, 1.0, JUNCTION);
  NodeT n3(1.0, 0.0, JUNCTION);
  
  EdgeT e1(n1, n2);
  EdgeT e1_rev(n2, n1);  // e1 with nodes reversed
  EdgeT e2(n1, n2);  // same as e1
  EdgeT e3(n1, n3);  // horizontal
  
  SECTION( "constructor for node 1" ){
    REQUIRE( n1.equal(e1.node1()) );
  }
  
  SECTION( "constructor for node 2" ){
    REQUIRE( n2.equal(e1.node2()) );
  }
  
  SECTION( "length" ){
    REQUIRE( approx(e1.length(), sqrt(2.0)) );
  }
  
  SECTION( "horizontal true" ){
    REQUIRE( e3.is_horizontal() );
  }
  
  SECTION( "horizontal false" ){
    REQUIRE( !e1.is_horizontal() );
  }
  
  SECTION( "equal with same nodes is true" ){
    REQUIRE( e1.equal(e2) );
  }
  
  SECTION( "equal with same nodes (reversed) is true" ){
    REQUIRE( e1.equal(e1_rev) );
  }
  
  SECTION( "equal with different nodes is false" ){
    REQUIRE( !e1.equal(e3) );
  }

}

TEST_CASE( "tests for EdgeT" , "[EdgeT]" ) {
  NodeT n1(0.0, 0.0, JUNCTION);
  NodeT n2(1.0, 1.0, JUNCTION);
  NodeT n3(1.0, 0.0, JUNCTION);
  
  EdgeT e1(n1, n2);
  EdgeT e1_rev(n2, n1);  // e1 with nodes reversed
  EdgeT e2(n1, n2);  // same as e1
  EdgeT e3(n1, n3);  // horizontal
  
  SECTION( "constructor for node 1" ){
    REQUIRE( n1.equal(e1.node1()) );
  }
  
  SECTION( "constructor for node 2" ){
    REQUIRE( n2.equal(e1.node2()) );
  }
  
  SECTION( "length" ){
    REQUIRE( approx(e1.length(), sqrt(2.0)) );
  }
  
  SECTION( "horizontal true" ){
    REQUIRE( e3.is_horizontal() );
  }
  
  SECTION( "horizontal false" ){
    REQUIRE( !e1.is_horizontal() );
  }
  
  SECTION( "equal with same nodes is true" ){
    REQUIRE( e1.equal(e2) );
  }
  
  SECTION( "equal with same nodes (reversed) is true" ){
    REQUIRE( e1.equal(e1_rev) );
  }
  
  SECTION( "equal with different nodes is false" ){
    REQUIRE( !e1.equal(e3) );
  }

}